@@ -378,6 +378,8 @@ macro_rules! _prepare_snapshot_for_redaction {
 /// simple values that do not implement the `Serialize` trait but does not
 /// permit redactions.
 ///
+/// Debug is called with `"{:#?}"`, which means this uses pretty-print.
+///
 /// The snapshot name is optional.
 #[macro_export]
 macro_rules! assert_debug_snapshot {
@@ -436,14 +438,20 @@ macro_rules! assert_display_snapshot {
 macro_rules! assert_snapshot {
     ($value:expr, @$snapshot:literal) => {
         $crate::assert_snapshot!(
-            $crate::_macro_support::ReferenceValue::Inline($snapshot),
+            $crate::_macro_support::ReferenceValue::Inline(
+                #[allow(clippy::needless_raw_string_hashes)]
+                $snapshot,
+            ),
             $value,
             stringify!($value)
         )
     };
     ($value:expr, $debug_expr:expr, @$snapshot:literal) => {
         $crate::assert_snapshot!(
-            $crate::_macro_support::ReferenceValue::Inline($snapshot),
+            $crate::_macro_support::ReferenceValue::Inline(
+                #[allow(clippy::needless_raw_string_hashes)]
+                $snapshot,
+            ),
             $value,
             $debug_expr
         )
@@ -538,17 +546,68 @@ macro_rules! with_settings {
 /// are deferred until the end of of it.  In other words this means that each snapshot
 /// assertion within the `glob!` block are reported.  It can be disabled by setting
 /// `INSTA_GLOB_FAIL_FAST` environment variable to `1`.
+///
+/// A three-argument version of this macro allows specifying a base directory
+/// for the glob to start in. This allows globbing in arbitrary directories,
+/// including parent directories:
+///
+/// ```
+/// # use insta::{assert_snapshot, glob, Settings};
+/// # let mut settings = Settings::clone_current();
+/// # settings.set_allow_empty_glob(true);
+/// # let _dropguard = settings.bind_to_scope();
+/// use std::fs;
+///
+/// glob!("../test_data", "inputs/*.txt", |path| {
+///     let input = fs::read_to_string(path).unwrap();
+///     assert_snapshot!(input.to_uppercase());
+/// });
+/// ```
 #[cfg(feature = "glob")]
 #[cfg_attr(docsrs, doc(cfg(feature = "glob")))]
 #[macro_export]
 macro_rules! glob {
-    ($glob:expr, $closure:expr) => {{
+    ($base_path:expr, $glob:expr, $closure:expr) => {{
+        use std::path::Path;
         let base = $crate::_macro_support::get_cargo_workspace(env!("CARGO_MANIFEST_DIR"))
-            .join(file!())
-            .parent()
-            .unwrap()
-            .canonicalize()
-            .unwrap_or_else(|e| panic!("failed to canonicalize insta::glob! base path: {}", e));
-        $crate::_macro_support::glob_exec(&base, $glob, $closure);
+            .join(Path::new(file!()).parent().unwrap())
+            .join($base_path)
+            .to_path_buf();
+
+        // we try to canonicalize but on some platforms (eg: wasm) that might not work, so
+        // we instead silently fall back.
+        let base = base.canonicalize().unwrap_or_else(|_| base);
+        $crate::_macro_support::glob_exec(env!("CARGO_MANIFEST_DIR"), &base, $glob, $closure);
+    }};
+
+    ($glob:expr, $closure:expr) => {{
+        insta::glob!(".", $glob, $closure)
     }};
 }
+
+/// Utility macro to permit a multi-snapshot run where all snapshots match.
+///
+/// Within this block, insta will allow an assertion to be run more than once
+/// (even inline) without generating another snapshot.  Instead it will assert
+/// that snapshot expressions visited more than once are matching.
+///
+/// ```rust
+/// insta::allow_duplicates! {
+///     for x in (0..10).step_by(2) {
+///         let is_even = x % 2 == 0;
+///         insta::assert_debug_snapshot!(is_even, @"true");
+///     }
+/// }
+/// ```
+///
+/// The first snapshot assertion will be used as a gold master and every further
+/// assertion will be checked against it.  If they don't match the assertion will
+/// fail.
+#[macro_export]
+macro_rules! allow_duplicates {
+    ($($x:tt)*) => {
+        $crate::_macro_support::with_allow_duplicates(|| {
+            $($x)*
+        })
+    }
+}
