@@ -1,5 +1,6 @@
 use std::borrow::Cow;
-use std::collections::BTreeMap;
+use std::cell::RefCell;
+use std::collections::{BTreeMap, BTreeSet};
 use std::error::Error;
 use std::fs;
 use std::io::Write;
@@ -8,10 +9,10 @@ use std::str;
 use std::sync::{Arc, Mutex};
 
 use crate::env::{
-    force_pass, force_update_snapshots, get_cargo_workspace, get_output_behavior,
-    get_snapshot_update_behavior, memoize_snapshot_file, OutputBehavior, SnapshotUpdate,
+    get_cargo_workspace, get_tool_config, memoize_snapshot_file, snapshot_update_behavior,
+    OutputBehavior, SnapshotUpdateBehavior, ToolConfig,
 };
-use crate::output::{print_snapshot_diff_with_title, print_snapshot_summary_with_title};
+use crate::output::SnapshotPrinter;
 use crate::settings::Settings;
 use crate::snapshot::{MetaData, PendingInlineSnapshot, Snapshot, SnapshotContents};
 use crate::utils::{path_to_storage, style};
@@ -21,6 +22,12 @@ lazy_static::lazy_static! {
         Mutex::new(BTreeMap::new());
     static ref TEST_NAME_CLASH_DETECTION: Mutex<BTreeMap<String, bool>> =
         Mutex::new(BTreeMap::new());
+    static ref INLINE_DUPLICATES: Mutex<BTreeSet<String>> =
+        Mutex::new(BTreeSet::new());
+}
+
+thread_local! {
+    static RECORDED_DUPLICATES: RefCell<Vec<BTreeMap<String, Snapshot>>> = RefCell::default()
 }
 
 // This macro is basically eprintln but without being captured and
@@ -124,6 +131,12 @@ fn detect_snapshot_name(
         }
     }
 
+    // The rest of the code just deals with duplicates, which we in some
+    // cases do not want to guard against.
+    if allow_duplicates() {
+        return Ok(name.to_string());
+    }
+
     // if the snapshot name clashes we need to increment a counter.
     // we really do not care about poisoning here.
     let mut counters = TEST_NAME_COUNTERS.lock().unwrap_or_else(|x| x.into_inner());
@@ -183,7 +196,7 @@ fn get_snapshot_filename(
                 write!(
                     &mut f,
                     "{}.snap",
-                    snapshot_name.replace('/', "__").replace('\\', "__")
+                    snapshot_name.replace(&['/', '\\'][..], "__")
                 )
                 .unwrap();
                 f
@@ -193,10 +206,12 @@ fn get_snapshot_filename(
 
 #[derive(Debug)]
 struct SnapshotAssertionContext<'a> {
+    tool_config: Arc<ToolConfig>,
     cargo_workspace: Arc<PathBuf>,
     module_path: &'a str,
     snapshot_name: Option<Cow<'a, str>>,
     snapshot_file: Option<PathBuf>,
+    duplication_key: Option<String>,
     old_snapshot: Option<Snapshot>,
     pending_snapshots_path: Option<PathBuf>,
     assertion_file: &'a str,
@@ -213,8 +228,10 @@ impl<'a> SnapshotAssertionContext<'a> {
         assertion_file: &'a str,
         assertion_line: u32,
     ) -> Result<SnapshotAssertionContext<'a>, Box<dyn Error>> {
+        let tool_config = get_tool_config(manifest_dir);
         let cargo_workspace = get_cargo_workspace(manifest_dir);
         let snapshot_name;
+        let mut duplication_key = None;
         let mut snapshot_file = None;
         let mut old_snapshot = None;
         let mut pending_snapshots_path = None;
@@ -228,6 +245,9 @@ impl<'a> SnapshotAssertionContext<'a> {
                         .unwrap()
                         .into(),
                 };
+                if allow_duplicates() {
+                    duplication_key = Some(format!("named:{}|{}", module_path, name));
+                }
                 let file = get_snapshot_filename(
                     module_path,
                     assertion_file,
@@ -243,6 +263,14 @@ impl<'a> SnapshotAssertionContext<'a> {
                 snapshot_file = Some(file);
             }
             ReferenceValue::Inline(contents) => {
+                if allow_duplicates() {
+                    duplication_key = Some(format!(
+                        "inline:{}|{}|{}",
+                        function_name, assertion_file, assertion_line
+                    ));
+                } else {
+                    prevent_inline_duplicate(function_name, assertion_file, assertion_line);
+                }
                 snapshot_name = detect_snapshot_name(function_name, module_path, true, is_doctest)
                     .ok()
                     .map(Cow::Owned);
@@ -266,6 +294,7 @@ impl<'a> SnapshotAssertionContext<'a> {
         };
 
         Ok(SnapshotAssertionContext {
+            tool_config,
             cargo_workspace,
             module_path,
             snapshot_name,
@@ -274,21 +303,16 @@ impl<'a> SnapshotAssertionContext<'a> {
             pending_snapshots_path,
             assertion_file,
             assertion_line,
+            duplication_key,
             is_doctest,
         })
     }
 
     /// Given a path returns the local path within the workspace.
     pub fn localize_path(&self, p: &Path) -> Option<PathBuf> {
-        self.cargo_workspace
-            .join(p)
-            .canonicalize()
-            .ok()
-            .and_then(|s| {
-                s.strip_prefix(self.cargo_workspace.as_path())
-                    .ok()
-                    .map(|x| x.to_path_buf())
-            })
+        let workspace = self.cargo_workspace.canonicalize().ok()?;
+        let p = self.cargo_workspace.join(p).canonicalize().ok()?;
+        p.strip_prefix(&workspace).ok().map(|x| x.to_path_buf())
     }
 
     /// Creates the new snapshot from input values.
@@ -339,19 +363,19 @@ impl<'a> SnapshotAssertionContext<'a> {
     pub fn update_snapshot(
         &self,
         new_snapshot: Snapshot,
-    ) -> Result<SnapshotUpdate, Box<dyn Error>> {
+    ) -> Result<SnapshotUpdateBehavior, Box<dyn Error>> {
         let unseen = self
             .snapshot_file
             .as_ref()
             .map_or(false, |x| fs::metadata(x).is_ok());
-        let should_print = get_output_behavior() != OutputBehavior::Nothing;
-        let snapshot_update = get_snapshot_update_behavior(unseen);
+        let should_print = self.tool_config.output_behavior() != OutputBehavior::Nothing;
+        let snapshot_update = snapshot_update_behavior(&self.tool_config, unseen);
 
         match snapshot_update {
-            SnapshotUpdate::InPlace => {
+            SnapshotUpdateBehavior::InPlace => {
                 if let Some(ref snapshot_file) = self.snapshot_file {
-                    new_snapshot.save(snapshot_file)?;
-                    if should_print {
+                    let saved = new_snapshot.save(snapshot_file)?;
+                    if should_print && saved {
                         elog!(
                             "{} {}",
                             if unseen {
@@ -374,17 +398,16 @@ impl<'a> SnapshotAssertionContext<'a> {
                     );
                 }
             }
-            SnapshotUpdate::NewFile => {
+            SnapshotUpdateBehavior::NewFile => {
                 if let Some(ref snapshot_file) = self.snapshot_file {
-                    let mut new_path = snapshot_file.to_path_buf();
-                    new_path.set_extension("snap.new");
-                    new_snapshot.save_new(&new_path)?;
-                    if should_print {
-                        elog!(
-                            "{} {}",
-                            style("stored new snapshot").green(),
-                            style(new_path.display()).cyan().underlined(),
-                        );
+                    if let Some(new_path) = new_snapshot.save_new(snapshot_file)? {
+                        if should_print {
+                            elog!(
+                                "{} {}",
+                                style("stored new snapshot").green(),
+                                style(new_path.display()).cyan().underlined(),
+                            );
+                        }
                     }
                 } else if self.is_doctest {
                     if should_print {
@@ -395,7 +418,19 @@ impl<'a> SnapshotAssertionContext<'a> {
                                 .bold(),
                         );
                     }
-                } else {
+
+                // special case for pending inline snapshots.  Here we really only want
+                // to write the contents if the snapshot contents changed as the metadata
+                // is not retained for inline snapshots.  This used to have different
+                // behavior in the past where we did indeed want to rewrite the snapshots
+                // entirely since we used to change the canonical snapshot format, but now
+                // this is significantly less likely to happen and seeing hundreds of unchanged
+                // inline snapshots in the review screen is not a lot of fun.
+                } else if self
+                    .old_snapshot
+                    .as_ref()
+                    .map_or(true, |x| x.contents() != new_snapshot.contents())
+                {
                     PendingInlineSnapshot::new(
                         Some(new_snapshot),
                         self.old_snapshot.clone(),
@@ -404,33 +439,45 @@ impl<'a> SnapshotAssertionContext<'a> {
                     .save(self.pending_snapshots_path.as_ref().unwrap())?;
                 }
             }
-            SnapshotUpdate::NoUpdate => {}
+            SnapshotUpdateBehavior::NoUpdate => {}
         }
 
         Ok(snapshot_update)
     }
 }
 
+fn prevent_inline_duplicate(function_name: &str, assertion_file: &str, assertion_line: u32) {
+    let key = format!("{}|{}|{}", function_name, assertion_file, assertion_line);
+    let mut set = INLINE_DUPLICATES.lock().unwrap();
+    if set.contains(&key) {
+        // drop the lock so we don't poison it
+        drop(set);
+        panic!(
+            "Insta does not allow inline snapshot assertions in loops. \
+            Wrap your assertions in allow_duplicates! to change this."
+        );
+    }
+    set.insert(key);
+}
+
 /// This prints the information about the snapshot
 fn print_snapshot_info(ctx: &SnapshotAssertionContext, new_snapshot: &Snapshot) {
-    match get_output_behavior() {
+    let mut printer = SnapshotPrinter::new(
+        ctx.cargo_workspace.as_path(),
+        ctx.old_snapshot.as_ref(),
+        new_snapshot,
+    );
+    printer.set_line(Some(ctx.assertion_line));
+    printer.set_snapshot_file(ctx.snapshot_file.as_deref());
+    printer.set_title(Some("Snapshot Summary"));
+    printer.set_show_info(true);
+    match ctx.tool_config.output_behavior() {
         OutputBehavior::Summary => {
-            print_snapshot_summary_with_title(
-                ctx.cargo_workspace.as_path(),
-                new_snapshot,
-                ctx.old_snapshot.as_ref(),
-                ctx.assertion_line,
-                ctx.snapshot_file.as_deref(),
-            );
+            printer.print();
         }
         OutputBehavior::Diff => {
-            print_snapshot_diff_with_title(
-                ctx.cargo_workspace.as_path(),
-                new_snapshot,
-                ctx.old_snapshot.as_ref(),
-                ctx.assertion_line,
-                ctx.snapshot_file.as_deref(),
-            );
+            printer.set_show_diff(true);
+            printer.print();
         }
         _ => {}
     }
@@ -449,7 +496,7 @@ macro_rules! print_or_panic {
 }
 
 /// Finalizes the assertion based on the update result.
-fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpdate) {
+fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpdateBehavior) {
     // if we are in glob mode, we want to adjust the finalization
     // so that we do not show the hints immediately.
     let fail_fast = {
@@ -468,8 +515,8 @@ fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpd
     };
 
     if fail_fast
-        && update_result == SnapshotUpdate::NewFile
-        && get_output_behavior() != OutputBehavior::Nothing
+        && update_result == SnapshotUpdateBehavior::NewFile
+        && ctx.tool_config.output_behavior() != OutputBehavior::Nothing
         && !ctx.is_doctest
     {
         println!(
@@ -478,8 +525,8 @@ fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpd
         );
     }
 
-    if update_result != SnapshotUpdate::InPlace && !force_pass() {
-        if fail_fast && get_output_behavior() != OutputBehavior::Nothing {
+    if update_result != SnapshotUpdateBehavior::InPlace && !ctx.tool_config.force_pass() {
+        if fail_fast && ctx.tool_config.output_behavior() != OutputBehavior::Nothing {
             println!(
                 "{hint}",
                 hint = style(
@@ -497,8 +544,8 @@ fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpd
             let mut stack = crate::glob::GLOB_STACK.lock().unwrap();
             if let Some(glob_collector) = stack.last_mut() {
                 glob_collector.failed += 1;
-                if update_result == SnapshotUpdate::NewFile
-                    && get_output_behavior() != OutputBehavior::Nothing
+                if update_result == SnapshotUpdateBehavior::NewFile
+                    && ctx.tool_config.output_behavior() != OutputBehavior::Nothing
                 {
                     glob_collector.show_insta_hint = true;
                 }
@@ -521,6 +568,56 @@ fn finalize_assertion(ctx: &SnapshotAssertionContext, update_result: SnapshotUpd
     }
 }
 
+fn record_snapshot_duplicate(
+    results: &mut BTreeMap<String, Snapshot>,
+    snapshot: &Snapshot,
+    ctx: &SnapshotAssertionContext,
+) {
+    let key = ctx.duplication_key.as_deref().unwrap();
+    if let Some(prev_snapshot) = results.get(key) {
+        if prev_snapshot.contents() != snapshot.contents() {
+            println!("Snapshots in allow-duplicates block do not match.");
+            let mut printer =
+                SnapshotPrinter::new(ctx.cargo_workspace.as_path(), Some(prev_snapshot), snapshot);
+            printer.set_line(Some(ctx.assertion_line));
+            printer.set_snapshot_file(ctx.snapshot_file.as_deref());
+            printer.set_title(Some("Differences in Block"));
+            printer.set_snapshot_hints("previous assertion", "current assertion");
+            if ctx.tool_config.output_behavior() == OutputBehavior::Diff {
+                printer.set_show_diff(true);
+            }
+            printer.print();
+            panic!(
+                "snapshot assertion for '{}' failed in line {}. Result \
+                    does not match previous snapshot in allow-duplicates block.",
+                ctx.snapshot_name.as_deref().unwrap_or("unnamed snapshot"),
+                ctx.assertion_line
+            );
+        }
+    } else {
+        results.insert(key.to_string(), snapshot.clone());
+    }
+}
+
+/// Do we allow recording of duplicates?
+fn allow_duplicates() -> bool {
+    RECORDED_DUPLICATES.with(|x| !x.borrow().is_empty())
+}
+
+/// Helper function to support perfect duplicate detection.
+pub fn with_allow_duplicates<R, F>(f: F) -> R
+where
+    F: FnOnce() -> R,
+{
+    RECORDED_DUPLICATES.with(|x| x.borrow_mut().push(BTreeMap::new()));
+    let rv = std::panic::catch_unwind(std::panic::AssertUnwindSafe(f));
+    RECORDED_DUPLICATES.with(|x| x.borrow_mut().pop().unwrap());
+    match rv {
+        Ok(rv) => rv,
+        Err(payload) => std::panic::resume_unwind(payload),
+    }
+}
+
 /// This function is invoked from the macros to run the main assertion logic.
 ///
 /// This will create the assertion context, run the main logic to assert
@@ -546,6 +643,7 @@ pub fn assert_snapshot(
         assertion_file,
         assertion_line,
     )?;
+    let tool_config = get_tool_config(manifest_dir);
 
     // apply filters if they are available
     #[cfg(feature = "filters")]
@@ -559,11 +657,20 @@ pub fn assert_snapshot(
         memoize_snapshot_file(snapshot_file);
     }
 
+    // If we allow assertion with duplicates, we record the duplicate now.  This will
+    // in itself fail the assertion if the previous visit of the same assertion macro
+    // did not yield the same result.
+    RECORDED_DUPLICATES.with(|x| {
+        if let Some(results) = x.borrow_mut().last_mut() {
+            record_snapshot_duplicate(results, &new_snapshot, &ctx);
+        }
+    });
+
     // pass if the snapshots are missing
     if ctx.old_snapshot.as_ref().map(|x| x.contents()) == Some(new_snapshot.contents()) {
         ctx.cleanup_passing()?;
 
-        if force_update_snapshots() {
+        if tool_config.force_update_snapshots() {
             ctx.update_snapshot(new_snapshot)?;
         }
     // otherwise print information and update snapshots.
